// A MouseJiggler for the Trinket attiny85 board
// If you install the TrinketHidCombo Lib into the Digistump Directory you
// may also use this jiggler on a DigiSpark
// There are Two Buttons:
// Button 1 on Port P0 (PB0) will toggle the jiggler on/off and
//          if pressed for more than 5 seconds go into the bootloader
//          the later is only tested on a DigiSpark
// Button 2 on Port P2 (PB2) will immediately send a SystemPowerOff keypress
//          which will shut down the Host OS
// LED is blinking a Heartbeat with Analog-like modulated Pulses
//     inverted if jiggler is off (Button 1)

#include <TrinketHidCombo.h>

#define PIXEL 3          // The maximum Move the jiggler does
#define MINWAIT 5000     // Wait minimum between two jiggles in ms
#define MAXWAIT 15000    // maximum between two jiggles

#define LEDPIN 1         // Onboard on most recent attiny85 boards (digispark compatible)
#define BUTTON 0         // Short to gnd with a button to toggle the jiggler state
// If jiggler is disabled the LED Heartbeat is inverted
#define BUTTONLONG 5000  // Pressing the Button for this time in ms will reset to bootloader
// Keep pressed until micronucleus flash starts
#define POWERBUTTON 2    // PowerOff Button

void setup() {
  randomSeed(analogRead(0));   // Initialize with some random noise from one analog input
  pinMode(LEDPIN, OUTPUT);
  pinMode(BUTTON, INPUT_PULLUP);
  pinMode(POWERBUTTON, INPUT_PULLUP);
  TrinketHidCombo.begin();
}

byte jiggler_enabled = 1;   // 0 = jiggling disabled, 1= jiggling enabled. Toggled via Button on Pin "BUTTON"

// Jiggles the Mouse when it's time to do so
// For that we compare the actual time (millis())
// and the time we last did something (lastrun)
// if the difference is more than wait (which varies between the steps)
// then do something
void jiggle() {
  static int state = 0;
  static int x, y;
  static long int wait = 0;
  static unsigned long lastrun = 0;
  unsigned long int nowms;
  nowms = millis();
  if (nowms - lastrun >= wait) {  // Next runtime reached?
    lastrun = nowms;
    switch (state) {
      case 0:
        if (jiggler_enabled == 1) {   // We only check this variable in this state (step) so that turning off
          x = random(-PIXEL, PIXEL);  // moves back the cursor to the old position first
          y = random(-PIXEL, PIXEL);  // We are using random() to pseudo-randomly move the cursor
          TrinketHidCombo.mouseMove(x, y, 0);
          wait = random(MINWAIT / 100, MAXWAIT / 100);
          state = 1;
        } else {
          wait = 500;
        }
        break;
      default:
        // move backwards;
        TrinketHidCombo.mouseMove(0 - x, 0 - y, 0);
        wait = random(MINWAIT, MAXWAIT);
        state = 0;
        break;
    }
  }
}

// Implementing a Heartbeat LED
// Scheduler works the same as in the jiggle()
void led() {
  static int state;
  static int ledvalue;
  static long int wait = 0;
  static unsigned long lastrun = 0;
  unsigned long int nowms;
  nowms = millis();
  if (nowms - lastrun >= wait) {  // Next runtime reached?
    lastrun = nowms;
    switch (state) {
      case 0:
      case 2:
        ledvalue += 24;
        if (ledvalue >= 255) {
          ledvalue = 255;
          state++;
        }
        wait = 15;
        break;
      case 1:
      case 3:
        ledvalue -= 24;
        if (ledvalue <= 0) {
          ledvalue = 0;
          state++;
        }
        wait = 15;
        break;
      default:
        wait = 500;
        state = 0;
        break;
    }
    if (jiggler_enabled == 0) { // Invert output if jiggler is disabled
      analogWrite(LEDPIN, 255 - ledvalue);
    } else {
      analogWrite(LEDPIN, ledvalue); // We are using Hardware PWM Output for the LED
    }
  }                                // So the Heartbeat is somewhat smooth
}

void reboot(void) {  // This code is from https://digistump.com/wiki/digispark/tricks
  noInterrupts(); // disable interrupts which could mess with changing prescaler
  CLKPR = 0b10000000; // enable prescaler speed change
  CLKPR = 0; // set prescaler to default (16mhz) mode required by bootloader
  void (*ptrToFunction)(); // allocate a function pointer
  ptrToFunction = 0x0000; // set function pointer to bootloader reset vector
  (*ptrToFunction)(); // jump to reset, which bounces in to bootloader
}

void button () {
  static unsigned long lasthigh = 0;
  static unsigned long lastedge = 0;
  unsigned long nowms;
  int state;
  static int oldstate = LOW;
  state = digitalRead(BUTTON);
  nowms = millis();
  if (state == LOW && oldstate == HIGH) {  // EDGE detected
    if ((nowms - lastedge) > 200) {        // But only if no Edge the last 200ms
      if (jiggler_enabled == 1) {
        jiggler_enabled = 0;
      } else {
        jiggler_enabled = 1;
      }
    }
    lastedge = nowms;
  }
  if (state == LOW) {
    if (lasthigh != 0) {                      // Only trigger if the pin was HIGH at least once
      if (nowms - lasthigh > BUTTONLONG) {    // State needs to be stable the last BUTTONLONG ms
        digitalWrite(LEDPIN, LOW);
        delay(150);
        for (state = 0; state < 5; state++) { // Blink 5 times to sigmal now is reset-time
          digitalWrite(LEDPIN, HIGH);         // We are re-using the variable state for the loop
          delay(150);                         // You need to keep the button pressed until reflash is
          digitalWrite(LEDPIN, LOW);          // starting if you have a ENTRY_JUMPER bootloader installed!
          delay(150);
        }
        reboot();                             // Reboot to bootloader
        lasthigh = 0;                         // Unnecessary as this code should never be reached
      }
    }
  }
  oldstate = state;
  if (state != LOW) {
    lasthigh = nowms;
  }
}

void powerbutton() {
  int state;
  state = digitalRead(POWERBUTTON);
  if (state == LOW) {
    TrinketHidCombo.pressSystemCtrlKey(SYSCTRLKEY_POWER);
  }
}

void loop() {
  button();                               // Check the Button
  powerbutton();                          // Pressing this Button will PowerOff Host
  jiggle();                               // Jiggle the mouse
  led();                                  // Heartbeat LED
  TrinketHidCombo.poll();
  delay(5);
}
